%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
function codeEach = HuffmanCodeSet(fnc)
%
% Generate the Huffman Code for this set of probabilities on the
% values
%
% TFB - 11/07
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % encode it using huffman coding
    codeEach = cell(1, length(fnc));
    nodesUnder = cell(1, length(fnc));
    curActive = ones([1, length(fnc)]);
    fnc = fnc ./ sum(fnc);
    
    % build the tree
    newProb = fnc;
    while ~any(newProb >= 0.9999)
        % Find 2 lowest probability (active) nodes
        [sortedN, sortedI] = sort(newProb);
        sortedI = sortedI(curActive(sortedI)==1);
        sortedN = newProb(sortedI);

        % Assign one of them a 0 and one of them a 1
        codeEach{sortedI(1)} = [codeEach{sortedI(1)} '0'];
        codeEach{sortedI(2)} = [codeEach{sortedI(2)} '1'];

        % Now mark them as not active
        curActive(sortedI(1:2)) = 0;

        % ... and add a new (active) node equal to their sum, with them as
        % children
        newProb(end+1) = sum(sortedN(1:2));
        nodesUnder{length(curActive)+1} = sortedI(1:2);
        curActive(end+1) = 1;
        codeEach{end+1} = [];
    end

    % unfold the tree
    for i=length(nodesUnder):-1:1
        if codeEach{i}
            for j=1:length(nodesUnder{i})
                codeEach{nodesUnder{i}(j)} = [codeEach{i} ...
                    codeEach{nodesUnder{i}(j)}];
            end
        end
    end
    codeEach = codeEach(1:length(fnc));
	
	
